==> 1.6.0.tar.gz: lib/src/widgets_live_keyed.dart <==
part of '../flutterx_live_data.dart';

/// LiveMap of LiveData
typedef KeyedLiveData<K, V> = LiveMap<K, LiveData<V>>;

/// A [Widget] whose content is built from the observed combination of [KeyedLiveData] and a [LiveData] key
class KeyedLiveDataBuilder<K, V> extends StatelessWidget {
  /// Data list which is observed
  final LiveData<K> dataKey;

  /// Data list which is observed
  final KeyedLiveData<K, V> data;

  /// Fallback LiveData when [data] does not contains [dataKey] if null an exception is thrown
  final LiveData<V>? fallback;

  /// Widget builder function
  final LiveWidgetBuilder<V> builder;

  /// Displayed when [data] is uninitialized, defaults to [SizedBox.shrink]
  final Widget? placeholder;

  /// Creates [builder] from [data]
  const KeyedLiveDataBuilder({
    Key? key,
    required this.dataKey,
    required this.data,
    this.fallback = const LateImmutableLiveData(),
    required this.builder,
    this.placeholder,
  }) : super(key: key);

  @override
  StatelessElement createElement() => _KeyedLiveDataBuilderElement<K, V>(this);

  @override
  Widget build(BuildContext context) {
    final data = (context as _KeyedLiveDataBuilderElement<K, V>)._data;
    return data != null && data.initialized ? builder(context, data._data) : (placeholder ?? const SizedBox.shrink());
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) => super.debugFillProperties(properties
    ..add(DiagnosticsProperty<LiveData<K>>('dataKey', dataKey))
    ..add(DiagnosticsProperty<KeyedLiveData<K, V>>('data', data))
    ..add(DiagnosticsProperty<LiveData<V>>('fallback', fallback))
    ..add(ObjectFlagProperty<LiveWidgetBuilder<V>>.has('builder', builder)));
}

class _KeyedLiveDataBuilderElement<K, V> extends StatelessElement
    with LiveDataBuilderElementMixin<KeyedLiveDataBuilder<K, V>> {
  LiveData<V>? _data;
  Disposable? _observer;

  _KeyedLiveDataBuilderElement(KeyedLiveDataBuilder<K, V> widget) : super(widget);

  @override
  bool dataChanged(KeyedLiveDataBuilder<K, V> oldWidget, KeyedLiveDataBuilder<K, V> newWidget) =>
      oldWidget.dataKey != newWidget.dataKey || oldWidget.data != newWidget.data;

  @override
  void link(KeyedLiveDataBuilder<K, V> widget) =>
      _observer = observe2Forever<K, Map<K, LiveData<V>>>(widget.dataKey, widget.data, (key, data) {
        final result = _data = data[key] ?? widget.fallback;
        if (result == null) throw StateError('data does not contains key $key and no fallback was provider');
        result.observeForever(onChanged, dispatcher: Dispatcher.postFrame);
      });

  @override
  void unlink(KeyedLiveDataBuilder<K, V> widget) {
    _data?.removeObserver(onChanged);
    _data = null;
    _observer?.dispose();
    _observer = null;
  }
}

==> 1.6.92.tar.gz: lib/src/widgets_live_keyed.dart <==
part of '../flutterx_live_data.dart';

/// LiveMap of LiveData
typedef KeyedLiveData<K, V> = LiveMap<K, LiveData<V>>;

/// A [Widget] whose content is built from the observed combination of [KeyedLiveData] and a [LiveData] key
class KeyedLiveDataBuilder<K, V> extends StatelessWidget {
  /// Data list which is observed
  final LiveData<K> dataKey;

  /// Data list which is observed
  final KeyedLiveData<K, V> data;

  /// Fallback LiveData when [data] does not contains [dataKey] if null an exception is thrown
  final LiveData<V>? fallback;

  /// Widget builder function
  final LiveWidgetBuilder<V> builder;

  /// Displayed when [data] is uninitialized, defaults to [SizedBox.shrink]
  final Widget? placeholder;

  /// Creates [builder] from [data]
  const KeyedLiveDataBuilder({
    super.key,
    required this.dataKey,
    required this.data,
    this.fallback = const LateImmutableLiveData(),
    required this.builder,
    this.placeholder,
  });

  @override
  StatelessElement createElement() => _KeyedLiveDataBuilderElement<K, V>(this);

  @override
  Widget build(BuildContext context) {
    final data = (context as _KeyedLiveDataBuilderElement<K, V>)._data;
    return data != null && data.initialized ? builder(context, data._data) : (placeholder ?? const SizedBox.shrink());
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) => super.debugFillProperties(properties
    ..add(DiagnosticsProperty<LiveData<K>>('dataKey', dataKey))
    ..add(DiagnosticsProperty<KeyedLiveData<K, V>>('data', data))
    ..add(DiagnosticsProperty<LiveData<V>>('fallback', fallback))
    ..add(ObjectFlagProperty<LiveWidgetBuilder<V>>.has('builder', builder)));
}

class _KeyedLiveDataBuilderElement<K, V> extends StatelessElement
    with LiveDataBuilderElementMixin<KeyedLiveDataBuilder<K, V>> {
  LiveData<V>? _data;
  Disposable? _observer;

  _KeyedLiveDataBuilderElement(KeyedLiveDataBuilder<K, V> super.widget);

  @override
  bool dataChanged(KeyedLiveDataBuilder<K, V> oldWidget, KeyedLiveDataBuilder<K, V> newWidget) =>
      oldWidget.dataKey != newWidget.dataKey || oldWidget.data != newWidget.data;

  @override
  void link(KeyedLiveDataBuilder<K, V> widget) =>
      _observer = observe2Forever<K, Map<K, LiveData<V>>>(widget.dataKey, widget.data, _observe);

  @override
  void unlink(KeyedLiveDataBuilder<K, V> widget) {
    _data?.removeObserver(onChanged);
    _data = null;
    _observer?.dispose();
    _observer = null;
  }

  void _observe(K key, Map<K, LiveData<V>> data) {
    final last = _data;
    final result = _data = data[key] ?? widget.fallback;
    if (last != null) {
      if (identical(last, result)) return;
      last.removeObserver(onChanged);
    }
    if (result == null) throw StateError('data does not contains key $key and no fallback was provider');
    result.observeForever(onChanged, dispatcher: Dispatcher.postFrame);
  }
}
